@@ -34,17 +34,38 @@ class Parser
     public static function lspci($debug = PSI_DEBUG)
     {
         $arrResults = array();
-        if (CommonFunctions::executeProgram("lspci", "", $strBuf, $debug)) {
+        if (CommonFunctions::executeProgram("lspci", (defined('PSI_SHOW_DEVICES_INFOS') && PSI_SHOW_DEVICES_INFOS)?"-m":"", $strBuf, $debug)) {
             $arrLines = preg_split("/\n/", $strBuf, -1, PREG_SPLIT_NO_EMPTY);
             foreach ($arrLines as $strLine) {
-                $arrParams = preg_split('/ /', trim($strLine), 2);
-                if (count($arrParams) == 2)
-                   $strName = $arrParams[1];
-                else
-                   $strName = "unknown";
-                $strName = preg_replace('/\(.*\)/', '', $strName);
                 $dev = new HWDevice();
-                $dev->setName($strName);
+                $arrParams = preg_split('/(\"? ")|(\" (?=-))/', trim($strLine));
+                if (defined('PSI_SHOW_DEVICES_INFOS') && PSI_SHOW_DEVICES_INFOS && ($cp = count($arrParams)) >= 6) {
+                    $arrParams[$cp-1] = trim($arrParams[$cp-1], '"'); // remove last "
+                    $dev->setName($arrParams[1].': '.$arrParams[2].' '.$arrParams[3]);
+                    if (preg_match('/^-/', $arrParams[4])) {
+                        if (($arrParams[5] !== "") && !preg_match('/^Unknown vendor/', $arrParams[5])) {
+                            $dev->setManufacturer(trim($arrParams[5]));
+                        }
+                        if (($arrParams[6] !== "") && !preg_match('/^Device /', $arrParams[6])) {
+                            $dev->setProduct(trim($arrParams[6]));
+                        }
+                    } else {
+                        if (($arrParams[4] !== "") && !preg_match('/^Unknown vendor/', $arrParams[4])) {
+                            $dev->setManufacturer(trim($arrParams[4]));
+                        }
+                        if (($arrParams[5] !== "") && !preg_match('/^Device /', $arrParams[5])) {
+                            $dev->setProduct(trim($arrParams[5]));
+                        }
+                    }
+                } else {
+                    $strLine=trim(preg_replace('/(")|( -\S+)/', '', $strLine));
+                    $arrParams = preg_split('/ /', trim($strLine), 2);
+                    if (count($arrParams) == 2)
+                        $strName = preg_replace('/\(rev\s[^\)]+\)/', '', $arrParams[1]);
+                    else
+                       $strName = "unknown";
+                    $dev->setName($strName);
+                }
                 $arrResults[] = $dev;
             }
         }
@@ -96,7 +117,7 @@ class Parser
                     $mount_parm[] = $parm;
                 }
             }
-        } elseif (CommonFunctions::rfts("/etc/mtab", $mount)) {
+        } elseif (CommonFunctions::rfts(((PSI_ROOT_FILESYSTEM === '')||(PSI_OS !== 'Linux'))?"/etc/mtab":"/proc/1/mounts", $mount)) {
             $mount = preg_split("/\n/", $mount, -1, PREG_SPLIT_NO_EMPTY);
             foreach ($mount as $mount_line) {
                 if (preg_match("/(\S+) (\S+) (\S+) (\S+) ([0-9]+) ([0-9]+)/", $mount_line, $mount_buf)) {
@@ -155,7 +176,7 @@ class Parser
                             foreach ($mount_parm as $mount_param) { //name and mountpoint find
                                 if (($mount_param['name']===trim($df_buf[0])) && ($mount_param['mountpoint']===$df_buf[5])) {
                                     $dev->setFsType($mount_param['fstype']);
-                                    if (PSI_SHOW_MOUNT_OPTION) {
+                                    if (PSI_SHOW_MOUNT_OPTION && (trim($mount_param['options'])!=="")) {
                                         if (PSI_SHOW_MOUNT_CREDENTIALS) {
                                             $dev->setOptions($mount_param['options']);
                                         } else {
@@ -183,7 +204,7 @@ class Parser
                             if ($notwas) foreach ($mount_parm as $mount_param) { //mountpoint find
                                 if ($mount_param['mountpoint']===$df_buf[5]) {
                                     $dev->setFsType($mount_param['fstype']);
-                                    if (PSI_SHOW_MOUNT_OPTION) {
+                                    if (PSI_SHOW_MOUNT_OPTION && (trim($mount_param['options'])!=="")) {
                                         if (PSI_SHOW_MOUNT_CREDENTIALS) {
                                             $dev->setOptions($mount_param['options']);
                                         } else {
@@ -224,45 +245,47 @@ class Parser
         } else {
             if (isset($mount_parm)) {
                 foreach ($mount_parm as $mount_param) {
-                    $total = disk_total_space($mount_param['mountpoint']);
-                    if (($mount_param['fstype'] != 'none') && ($total > 0)) {
-                        $dev = new DiskDevice();
-                        $dev->setName($mount_param['name']);
-                        $dev->setFsType($mount_param['fstype']);
+                    if (is_dir($mount_param['mountpoint'])) {
+                        $total = disk_total_space($mount_param['mountpoint']);
+                        if (($mount_param['fstype'] != 'none') && ($total > 0)) {
+                            $dev = new DiskDevice();
+                            $dev->setName($mount_param['name']);
+                            $dev->setFsType($mount_param['fstype']);
 
-                        if (PSI_SHOW_MOUNT_POINT) $dev->setMountPoint($mount_param['mountpoint']);
+                            if (PSI_SHOW_MOUNT_POINT) $dev->setMountPoint($mount_param['mountpoint']);
 
-                        $dev->setTotal($total);
-                        $free = disk_free_space($mount_param['mountpoint']);
-                        if ($free > 0) {
-                            $dev->setFree($free);
-                        } else {
-                            $free = 0;
-                        }
-                        if ($total > $free) $dev->setUsed($total - $free);
-
-                        if (PSI_SHOW_MOUNT_OPTION) {
-                            if (PSI_SHOW_MOUNT_CREDENTIALS) {
-                                $dev->setOptions($mount_param['options']);
+                            $dev->setTotal($total);
+                            $free = disk_free_space($mount_param['mountpoint']);
+                            if ($free > 0) {
+                                $dev->setFree($free);
                             } else {
-                                $mpo=$mount_param['options'];
-
-                                $mpo=preg_replace('/(^guest,)|(^guest$)|(,guest$)/i', '', $mpo);
-                                $mpo=preg_replace('/,guest,/i', ',', $mpo);
-
-                                $mpo=preg_replace('/(^user=[^,]*,)|(^user=[^,]*$)|(,user=[^,]*$)/i', '', $mpo);
-                                $mpo=preg_replace('/,user=[^,]*,/i', ',', $mpo);
-
-                                $mpo=preg_replace('/(^username=[^,]*,)|(^username=[^,]*$)|(,username=[^,]*$)/i', '', $mpo);
-                                $mpo=preg_replace('/,username=[^,]*,/i', ',', $mpo);
-
-                                $mpo=preg_replace('/(^password=[^,]*,)|(^password=[^,]*$)|(,password=[^,]*$)/i', '', $mpo);
-                                $mpo=preg_replace('/,password=[^,]*,/i', ',', $mpo);
-
-                                $dev->setOptions($mpo);
+                                $free = 0;
                             }
+                            if ($total > $free) $dev->setUsed($total - $free);
+
+                            if (PSI_SHOW_MOUNT_OPTION) {
+                                if (PSI_SHOW_MOUNT_CREDENTIALS) {
+                                    $dev->setOptions($mount_param['options']);
+                                } else {
+                                    $mpo=$mount_param['options'];
+
+                                    $mpo=preg_replace('/(^guest,)|(^guest$)|(,guest$)/i', '', $mpo);
+                                    $mpo=preg_replace('/,guest,/i', ',', $mpo);
+
+                                    $mpo=preg_replace('/(^user=[^,]*,)|(^user=[^,]*$)|(,user=[^,]*$)/i', '', $mpo);
+                                    $mpo=preg_replace('/,user=[^,]*,/i', ',', $mpo);
+
+                                    $mpo=preg_replace('/(^username=[^,]*,)|(^username=[^,]*$)|(,username=[^,]*$)/i', '', $mpo);
+                                    $mpo=preg_replace('/,username=[^,]*,/i', ',', $mpo);
+
+                                    $mpo=preg_replace('/(^password=[^,]*,)|(^password=[^,]*$)|(,password=[^,]*$)/i', '', $mpo);
+                                    $mpo=preg_replace('/,password=[^,]*,/i', ',', $mpo);
+
+                                    $dev->setOptions($mpo);
+                                }
+                            }
+                            $arrResult[] = $dev;
                         }
-                        $arrResult[] = $dev;
                     }
                 }
             }
