@@ -30,9 +30,9 @@ class Darwin extends BSDCommon
     /**
      * define the regexp for log parser
      */
-    /* public function __construct()
+    /* public function __construct($blockname = false)
     {
-        parent::__construct();
+        parent::__construct($blockname);
         $this->error->addWarning("The Darwin version of phpSysInfo is a work in progress, some things currently don't work!");
         $this->setCPURegExp1("/CPU: (.*) \((.*)-MHz (.*)\)/");
         $this->setCPURegExp2("/(.*) ([0-9]+) ([0-9]+) ([0-9]+) ([0-9]+)/");
@@ -89,25 +89,6 @@ class Darwin extends BSDCommon
         }
     }
 
-    /**
-     * UpTime
-     * time the system is running
-     *
-     * @return void
-     */
-    private function _uptime()
-    {
-        if (CommonFunctions::executeProgram('sysctl', '-n kern.boottime', $a, PSI_DEBUG)) {
-            $tmp = explode(" ", $a);
-            if ($tmp[0]=="{") { /* kern.boottime= { sec = 1096732600, usec = 885425 } Sat Oct 2 10:56:40 2004 */
-              $data = trim($tmp[3], ",");
-              $this->sys->setUptime(time() - $data);
-            } else { /* kern.boottime= 1096732600 */
-              $this->sys->setUptime(time() - $a);
-            }
-        }
-    }
-
     /**
      * get CPU information
      *
@@ -116,12 +97,12 @@ class Darwin extends BSDCommon
     protected function cpuinfo()
     {
         $dev = new CpuDevice();
-        if (CommonFunctions::executeProgram('hostinfo', '| grep "Processor type"', $buf, PSI_DEBUG)) {
-            $dev->setModel(preg_replace('/Processor type: /', '', $buf));
+        if (CommonFunctions::executeProgram('hostinfo', '', $buf, PSI_DEBUG) && ($buf !== '') && preg_match('/^Processor type:[ ]+(.+)$/m', $buf, $proc) && (($proc[1] = trim($proc[1])) !== '')) {
+            $dev->setModel($proc[1]);
             $buf=$this->grabkey('hw.model');
-            if (!is_null($buf) && (trim($buf) != "")) {
+            if (($buf !== null) && (trim($buf) != "")) {
                 $this->sys->setMachine(trim($buf));
-                if (CommonFunctions::rfts(APP_ROOT.'/data/ModelTranslation.txt', $buffer)) {
+                if (CommonFunctions::rftsdata('ModelTranslation.txt', $buffer)) {
                     $buffer = preg_split("/\n/", $buffer, -1, PREG_SPLIT_NO_EMPTY);
                     foreach ($buffer as $line) {
                         $ar_buf = preg_split("/:/", $line, 3);
@@ -134,12 +115,12 @@ class Darwin extends BSDCommon
                 }
             }
             $buf=$this->grabkey('machdep.cpu.brand_string');
-            if (!is_null($buf) && (trim($buf) != "") &&
+            if (($buf !== null) && (trim($buf) != "") &&
                  ((trim($buf) != "i486 (Intel 80486)") || ($dev->getModel() == ""))) {
                 $dev->setModel(trim($buf));
             }
             $buf=$this->grabkey('machdep.cpu.features');
-            if (!is_null($buf) && (trim($buf) != "")) {
+            if (($buf !== null) && (trim($buf) != "")) {
                 if (preg_match("/ VMX/", $buf)) {
                     $dev->setVirt("vmx");
                 } elseif (preg_match("/ SVM/", $buf)) {
@@ -151,17 +132,21 @@ class Darwin extends BSDCommon
         $dev->setBusSpeed(round($this->grabkey('hw.busfrequency') / 1000000));
         $bufn=$this->grabkey('hw.cpufrequency_min');
         $bufx=$this->grabkey('hw.cpufrequency_max');
-        if (!is_null($bufn) && (trim($bufn) != "") && !is_null($bufx) && (trim($bufx) != "") && ($bufn != $bufx)) {
+        if (($bufn !== null) && (trim($bufn) != "") && ($bufx !== null) && (trim($bufx) != "") && ($bufn != $bufx)) {
             $dev->setCpuSpeedMin(round($bufn / 1000000));
             $dev->setCpuSpeedMax(round($bufx / 1000000));
         }
         $buf=$this->grabkey('hw.l2cachesize');
-        if (!is_null($buf) && (trim($buf) != "")) {
+        if ($buf !== "") {
             $dev->setCache(round($buf));
         }
         $ncpu = $this->grabkey('hw.ncpu');
-        if (is_null($ncpu) || (trim($ncpu) == "") || (!($ncpu >= 1)))
+        if (($ncpu === "") || !($ncpu >= 1)) {
             $ncpu = 1;
+        }
+        if (($ncpu == 1) && PSI_LOAD_BAR) {
+            $dev->setLoad($this->cpuusage());
+        }
         for ($ncpu ; $ncpu > 0 ; $ncpu--) {
             $this->sys->setCpus($dev);
         }
@@ -238,6 +223,18 @@ class Darwin extends BSDCommon
                     if (!preg_match('/"USB Product Name" = "([^"]*)"/', $line, $ar_buf))
                        $ar_buf = preg_split("/[\s@]+/", $line, 19);
                     $dev->setName(trim($ar_buf[1]));
+                    if (defined('PSI_SHOW_DEVICES_INFOS') && PSI_SHOW_DEVICES_INFOS) {
+                        if (preg_match('/"USB Vendor Name" = "([^"]*)"/', $line, $ar_buf)) {
+                            $dev->setManufacturer(trim($ar_buf[1]));
+                        }
+                        if (preg_match('/"USB Product Name" = "([^"]*)"/', $line, $ar_buf)) {
+                            $dev->setProduct(trim($ar_buf[1]));
+                        }
+                        if (defined('PSI_SHOW_DEVICES_SERIAL') && PSI_SHOW_DEVICES_SERIAL
+                           && preg_match('/"USB Serial Number" = "([^"]*)"/', $line, $ar_buf)) {
+                            $dev->setSerial(trim($ar_buf[1]));
+                        }
+                    }
                     $this->sys->setUsbDevices($dev);
         }
     }
@@ -267,60 +264,63 @@ class Darwin extends BSDCommon
      */
     protected function memory()
     {
-        $s = $this->grabkey('hw.memsize');
-        if (CommonFunctions::executeProgram('vm_stat', '', $pstat, PSI_DEBUG)) {
-            // calculate free memory from page sizes (each page = 4096)
-            if (preg_match('/^Pages free:\s+(\S+)/m', $pstat, $free_buf)) {
-                if (preg_match('/^Anonymous pages:\s+(\S+)/m', $pstat, $anon_buf)
-                   && preg_match('/^Pages wired down:\s+(\S+)/m', $pstat, $wire_buf)
-                   && preg_match('/^File-backed pages:\s+(\S+)/m', $pstat, $fileb_buf)) {
-                        // OS X 10.9 or never
-                        $this->sys->setMemFree($free_buf[1] * 4 * 1024);
-                        $this->sys->setMemApplication(($anon_buf[1]+$wire_buf[1]) * 4 * 1024);
-                        $this->sys->setMemCache($fileb_buf[1] * 4 * 1024);
-                        if (preg_match('/^Pages occupied by compressor:\s+(\S+)/m', $pstat, $compr_buf)) {
-                            $this->sys->setMemBuffer($compr_buf[1] * 4 * 1024);
-                        }
-                } else {
-                    if (preg_match('/^Pages speculative:\s+(\S+)/m', $pstat, $spec_buf)) {
-                        $this->sys->setMemFree(($free_buf[1]+$spec_buf[1]) * 4 * 1024);
+        if (($s = $this->grabkey('hw.memsize')) > 0) {
+            $this->sys->setMemTotal($s);
+            if (CommonFunctions::executeProgram('vm_stat', '', $pstat, PSI_DEBUG)) {
+                // calculate free memory from page sizes (each page = 4096)
+                if (preg_match('/^Pages free:\s+(\S+)/m', $pstat, $free_buf)) {
+                    if (preg_match('/^Anonymous pages:\s+(\S+)/m', $pstat, $anon_buf)
+                       && preg_match('/^Pages wired down:\s+(\S+)/m', $pstat, $wire_buf)
+                       && preg_match('/^File-backed pages:\s+(\S+)/m', $pstat, $fileb_buf)) {
+                            // OS X 10.9 or never
+                            $this->sys->setMemFree($free_buf[1] * 4 * 1024);
+                            $this->sys->setMemApplication(($anon_buf[1]+$wire_buf[1]) * 4 * 1024);
+                            $this->sys->setMemCache($fileb_buf[1] * 4 * 1024);
+                            if (preg_match('/^Pages occupied by compressor:\s+(\S+)/m', $pstat, $compr_buf)) {
+                                $this->sys->setMemBuffer($compr_buf[1] * 4 * 1024);
+                            }
                     } else {
-                        $this->sys->setMemFree($free_buf[1] * 4 * 1024);
-                    }
-                    $appMemory = 0;
-                    if (preg_match('/^Pages wired down:\s+(\S+)/m', $pstat, $wire_buf)) {
-                        $appMemory += $wire_buf[1] * 4 * 1024;
-                    }
-                    if (preg_match('/^Pages active:\s+(\S+)/m', $pstat, $active_buf)) {
-                        $appMemory += $active_buf[1] * 4 * 1024;
-                    }
-                    $this->sys->setMemApplication($appMemory);
+                        if (preg_match('/^Pages speculative:\s+(\S+)/m', $pstat, $spec_buf)) {
+                            $this->sys->setMemFree(($free_buf[1]+$spec_buf[1]) * 4 * 1024);
+                        } else {
+                            $this->sys->setMemFree($free_buf[1] * 4 * 1024);
+                        }
+                        $appMemory = 0;
+                        if (preg_match('/^Pages wired down:\s+(\S+)/m', $pstat, $wire_buf)) {
+                            $appMemory += $wire_buf[1] * 4 * 1024;
+                        }
+                        if (preg_match('/^Pages active:\s+(\S+)/m', $pstat, $active_buf)) {
+                            $appMemory += $active_buf[1] * 4 * 1024;
+                        }
+                        $this->sys->setMemApplication($appMemory);
 
-                    if (preg_match('/^Pages inactive:\s+(\S+)/m', $pstat, $inactive_buf)) {
-                        $this->sys->setMemCache($inactive_buf[1] * 4 * 1024);
+                        if (preg_match('/^Pages inactive:\s+(\S+)/m', $pstat, $inactive_buf)) {
+                            $this->sys->setMemCache($inactive_buf[1] * 4 * 1024);
+                        }
                     }
+                } else {
+                    $lines = preg_split("/\n/", $pstat, -1, PREG_SPLIT_NO_EMPTY);
+                    $ar_buf = preg_split("/\s+/", $lines[1], 19);
+                    $this->sys->setMemFree($ar_buf[2] * 4 * 1024);
                 }
-            } else {
-                $lines = preg_split("/\n/", $pstat, -1, PREG_SPLIT_NO_EMPTY);
-                $ar_buf = preg_split("/\s+/", $lines[1], 19);
-                $this->sys->setMemFree($ar_buf[2] * 4 * 1024);
+                $this->sys->setMemUsed($this->sys->getMemTotal() - $this->sys->getMemFree());
             }
 
-            $this->sys->setMemTotal($s);
-            $this->sys->setMemUsed($this->sys->getMemTotal() - $this->sys->getMemFree());
-
-            if (CommonFunctions::executeProgram('sysctl', 'vm.swapusage | colrm 1 22', $swapBuff, PSI_DEBUG)) {
-                $swap1 = preg_split('/M/', $swapBuff);
-                $swap2 = preg_split('/=/', $swap1[1]);
-                $swap3 = preg_split('/=/', $swap1[2]);
-                $dev = new DiskDevice();
-                $dev->setName('SWAP');
-                $dev->setMountPoint('SWAP');
-                $dev->setFsType('swap');
-                $dev->setTotal($swap1[0] * 1024 * 1024);
-                $dev->setUsed($swap2[1] * 1024 * 1024);
-                $dev->setFree($swap3[1] * 1024 * 1024);
-                $this->sys->setSwapDevices($dev);
+            if (($swap = $this->grabkey("vm.swapusage")) > 0) {
+                $swap0 = preg_split('/M/', $swapBuff);
+                $swap1 = preg_split('/=/', $swap0[0]);
+                $swap2 = preg_split('/=/', $swap0[1]);
+                $swap3 = preg_split('/=/', $swap0[2]);
+                if (($swap=str_replace(',', '.', trim($swap1[1]))) > 0) {
+                    $dev = new DiskDevice();
+                    $dev->setName('SWAP');
+                    $dev->setMountPoint('SWAP');
+                    $dev->setFsType('swap');
+                    $dev->setTotal($swap * 1024 * 1024);
+                    $dev->setUsed(str_replace(',', '.', trim($swap2[1])) * 1024 * 1024);
+                    $dev->setFree(str_replace(',', '.', trim($swap3[1])) * 1024 * 1024);
+                    $this->sys->setSwapDevices($dev);
+                }
             }
         }
     }
@@ -401,28 +401,37 @@ class Darwin extends BSDCommon
     protected function distro()
     {
         $this->sys->setDistributionIcon('Darwin.png');
-        if (!CommonFunctions::executeProgram('system_profiler', 'SPSoftwareDataType', $buffer, PSI_DEBUG)) {
+        if ((!CommonFunctions::executeProgram('system_profiler', 'SPSoftwareDataType', $buffer, PSI_DEBUG) || !preg_match('/\n\s*System Version:/', $buffer))
+           && (!CommonFunctions::executeProgram('sw_vers', '', $buffer, PSI_DEBUG) || !preg_match('/^ProductName:/', $buffer))) {
             parent::distro();
         } else {
-            $arrBuff = preg_split("/\n/", $buffer, -1, PREG_SPLIT_NO_EMPTY);
-            foreach ($arrBuff as $line) {
-                $arrLine = preg_split("/:/", $line, -1, PREG_SPLIT_NO_EMPTY);
-                if (trim($arrLine[0]) === "System Version") {
-                    $distro = trim($arrLine[1]);
-
-                    if (preg_match('/(^Mac OS)|(^OS X)|(^macOS)/', $distro)) {
-                        $this->sys->setDistributionIcon('Apple.png');
-                        if (preg_match('/((^Mac OS X Server)|(^Mac OS X)|(^OS X Server)|(^OS X)|(^macOS Server)|(^macOS)) (\d+\.\d+)/', $distro, $ver)
-                            && ($list = @parse_ini_file(APP_ROOT."/data/osnames.ini", true))
-                            && isset($list['OS X'][$ver[6]])) {
-                            $distro.=' '.$list['OS X'][$ver[6]];
+            $distro_tmp = preg_split("/\n/", $buffer, -1, PREG_SPLIT_NO_EMPTY);
+            foreach ($distro_tmp as $info) {
+                $info_tmp = preg_split('/:/', $info, 2);
+                if (isset($distro_tmp[0]) && ($distro_tmp[0] !== null) && (trim($distro_tmp[0]) != "") &&
+                     isset($distro_tmp[1]) && ($distro_tmp[1] !== null) && (trim($distro_tmp[1]) != "")) {
+                    $distro_arr[trim($info_tmp[0])] = trim($info_tmp[1]);
+                }
+            }
+            if (isset($distro_arr['ProductName']) && isset($distro_arr['ProductVersion']) && isset($distro_arr['BuildVersion'])) {
+                $distro_arr['System Version'] = $distro_arr['ProductName'].' '.$distro_arr['ProductVersion'].' ('.$distro_arr['BuildVersion'].')';
+            }
+            if (isset($distro_arr['System Version'])) {
+                $distro = $distro_arr['System Version'];
+                if (preg_match('/^Mac OS |^OS X |^macOS |^iPhone OS |^Mac OS$|^OS X$|^macOS$|^iPhone OS$/', $distro)) {
+                    $this->sys->setDistributionIcon('Apple.png');
+                    if (preg_match('/(^Mac OS X Server|^Mac OS X|^OS X Server|^OS X|^macOS Server|^macOS) ((\d+)\.\d+)/', $distro, $ver)
+                        && ($list = @parse_ini_file(PSI_APP_ROOT."/data/osnames.ini", true))) {
+                        if (isset($list['macOS'][$ver[2]])) {
+                            $distro.=' '.$list['macOS'][$ver[2]];
+                        } elseif (isset($list['macOS'][$ver[3]])) {
+                            $distro.=' '.$list['macOS'][$ver[3]];
                         }
                     }
-
-                    $this->sys->setDistribution($distro);
-
-                    return;
                 }
+                $this->sys->setDistribution($distro);
+            } else {
+                parent::distro();
             }
         }
     }
@@ -462,19 +471,18 @@ class Darwin extends BSDCommon
      *
      * @see PSI_Interface_OS::build()
      *
-     * @return Void
+     * @return void
      */
     public function build()
     {
         parent::build();
-        if (!defined('PSI_ONLY') || PSI_ONLY==='vitals') {
-            $this->_uptime();
+        if (!$this->blockname || $this->blockname==='vitals') {
             $this->_processes();
         }
-        if (!defined('PSI_ONLY') || PSI_ONLY==='hardware') {
+        if (!$this->blockname || $this->blockname==='hardware') {
             $this->_tb();
         }
-        if (!defined('PSI_ONLY') || PSI_ONLY==='network') {
+        if (!$this->blockname || $this->blockname==='network') {
             $this->_network();
         }
     }
