@@ -88,6 +88,10 @@ class MBInfo
      */
     public function getMbFan()
     {
+        if (defined('PSI_SORT_SENSORS_LIST') && PSI_SORT_SENSORS_LIST) {
+            usort($this->_mbFan, array('CommonFunctions', 'name_natural_compare'));
+        }
+
         return $this->_mbFan;
     }
 
@@ -114,6 +118,10 @@ class MBInfo
      */
     public function getMbTemp()
     {
+        if (defined('PSI_SORT_SENSORS_LIST') && PSI_SORT_SENSORS_LIST) {
+            usort($this->_mbTemp, array('CommonFunctions', 'name_natural_compare'));
+        }
+
         return $this->_mbTemp;
     }
 
@@ -140,6 +148,10 @@ class MBInfo
      */
     public function getMbVolt()
     {
+        if (defined('PSI_SORT_SENSORS_LIST') && PSI_SORT_SENSORS_LIST) {
+            usort($this->_mbVolt, array('CommonFunctions', 'name_natural_compare'));
+        }
+
         return $this->_mbVolt;
     }
 
@@ -166,6 +178,10 @@ class MBInfo
      */
     public function getMbPower()
     {
+        if (defined('PSI_SORT_SENSORS_LIST') && PSI_SORT_SENSORS_LIST) {
+            usort($this->_mbPower, array('CommonFunctions', 'name_natural_compare'));
+        }
+
         return $this->_mbPower;
     }
 
@@ -192,6 +208,10 @@ class MBInfo
      */
     public function getMbCurrent()
     {
+        if (defined('PSI_SORT_SENSORS_LIST') && PSI_SORT_SENSORS_LIST) {
+            usort($this->_mbCurrent, array('CommonFunctions', 'name_natural_compare'));
+        }
+
         return $this->_mbCurrent;
     }
 
@@ -218,6 +238,10 @@ class MBInfo
      */
     public function getMbOther()
     {
+        if (defined('PSI_SORT_SENSORS_LIST') && PSI_SORT_SENSORS_LIST) {
+            usort($this->_mbOther, array('CommonFunctions', 'name_natural_compare'));
+        }
+
         return $this->_mbOther;
     }
 
