@@ -65,13 +65,6 @@ class XML
      */
     private $_plugin = '';
 
-    /**
-     * generate a xml for a plugin or for the main app
-     *
-     * @var boolean
-     */
-    private $_plugin_request = false;
-
     /**
      * generate the entire xml with all plugins or only a part of the xml (main or plugin)
      *
@@ -90,23 +83,23 @@ class XML
      *
      * @return void
      */
-    public function __construct($complete = false, $pluginname = "")
+    public function __construct($complete = false, $pluginname = "", $blockname = false)
     {
         $this->_errors = PSI_Error::singleton();
-        if ($pluginname == "") {
-            $this->_plugin_request = false;
-            $this->_plugin = '';
-        } else {
-            $this->_plugin_request = true;
-            $this->_plugin = $pluginname;
-        }
+        $this->_plugin = $pluginname;
         if ($complete) {
             $this->_complete_request = true;
         } else {
             $this->_complete_request = false;
         }
-        $os = PSI_OS;
-        $this->_sysinfo = new $os();
+        if (defined('PSI_EMU_PORT')) {
+            $os = 'SSH';
+        } elseif (defined('PSI_EMU_HOSTNAME')) {
+            $os = 'WINNT';
+        } else {
+            $os = PSI_OS;
+        }
+        $this->_sysinfo = new $os($blockname);
         $this->_plugins = CommonFunctions::getPlugins();
         $this->_xmlbody();
     }
@@ -169,7 +162,14 @@ class XML
                 }
             }
         }
-        $vitals->addAttribute('OS', PSI_OS);
+
+        if (($os = $this->_sys->getOS()) == 'Android') {
+            $vitals->addAttribute('OS', 'Linux');
+        } elseif ($os == 'GNU') {
+            $vitals->addAttribute('OS', 'Hurd');
+        } else {
+            $vitals->addAttribute('OS', $os);
+        }
     }
 
     /**
@@ -193,11 +193,42 @@ class XML
             }
         }
         foreach ($this->_sys->getNetDevices() as $dev) {
-            if (!in_array(trim($dev->getName()), $hideDevices)) {
+            if (defined('PSI_HIDE_NETWORK_INTERFACE_REGEX') && PSI_HIDE_NETWORK_INTERFACE_REGEX) {
+                $hide = false;
+                foreach ($hideDevices as $hidedev) {
+                    if (preg_match('/^'.$hidedev.'$/', trim($dev->getName()))) {
+                        $hide = true;
+                        break;
+                    }
+                }
+            } else {
+                $hide =in_array(trim($dev->getName()), $hideDevices);
+            }
+            if (!$hide) {
                 $device = $network->addChild('NetDevice');
                 $device->addAttribute('Name', $dev->getName());
-                $device->addAttribute('RxBytes', $dev->getRxBytes());
-                $device->addAttribute('TxBytes', $dev->getTxBytes());
+                $rxbytes = $dev->getRxBytes();
+                $txbytes = $dev->getTxBytes();
+                $device->addAttribute('RxBytes', $rxbytes);
+                $device->addAttribute('TxBytes', $txbytes);
+                if (defined('PSI_SHOW_NETWORK_ACTIVE_SPEED') && PSI_SHOW_NETWORK_ACTIVE_SPEED) {
+                    if (($rxbytes == 0) && ($txbytes == 0)) {
+                        $rxrate = $dev->getRxRate();
+                        $txrate = $dev->getTxRate();
+                        if (($rxrate !== null) || ($txrate !== null)) {
+                            if ($rxrate !== null) {
+                                $device->addAttribute('RxRate', $rxrate);
+                            } else {
+                                $device->addAttribute('RxRate', 0);
+                            }
+                            if ($txrate !== null) {
+                                $device->addAttribute('TxRate', $txrate);
+                            } else {
+                                $device->addAttribute('TxRate', 0);
+                            }
+                        }
+                    }
+                }
                 $device->addAttribute('Err', $dev->getErrors());
                 $device->addAttribute('Drops', $dev->getDrops());
                 if (defined('PSI_SHOW_NETWORK_INFOS') && PSI_SHOW_NETWORK_INFOS && $dev->getInfo())
@@ -214,70 +245,81 @@ class XML
     private function _buildHardware()
     {
         $hardware = $this->_xml->addChild('Hardware');
-        if ($this->_sys->getMachine() != "") {
-            $hardware->addAttribute('Name', $this->_sys->getMachine());
-        }
-        $pci = null;
-        foreach (System::removeDupsAndCount($this->_sys->getPciDevices()) as $dev) {
-            if ($pci === null) $pci = $hardware->addChild('PCI');
-            $tmp = $pci->addChild('Device');
-            $tmp->addAttribute('Name', $dev->getName());
-            $tmp->addAttribute('Count', $dev->getCount());
-        }
-        $usb = null;
-        foreach (System::removeDupsAndCount($this->_sys->getUsbDevices()) as $dev) {
-            if ($usb === null) $usb = $hardware->addChild('USB');
-            $tmp = $usb->addChild('Device');
-            $tmp->addAttribute('Name', $dev->getName());
-            $tmp->addAttribute('Count', $dev->getCount());
-        }
-        $ide = null;
-        foreach (System::removeDupsAndCount($this->_sys->getIdeDevices()) as $dev) {
-            if ($ide === null) $ide = $hardware->addChild('IDE');
-            $tmp = $ide->addChild('Device');
-            $tmp->addAttribute('Name', $dev->getName());
-            $tmp->addAttribute('Count', $dev->getCount());
-            if ($dev->getCapacity() !== null) {
-                $tmp->addAttribute('Capacity', $dev->getCapacity());
+        if (($machine = $this->_sys->getMachine()) != "") {
+            $machine = trim(preg_replace("/\s+/", " ", preg_replace("/^\s*[\/,]*/", "", preg_replace("/\/\s+,/", "/,", $machine)))); // remove leading slash or comma and unnecessary spaces
+            if (preg_match('/, BIOS .*$/', $machine, $mbuf, PREG_OFFSET_CAPTURE)) {
+                $comapos = $mbuf[0][1];
+                $endstr = $mbuf[0][0];
+                $offset = 0;
+                while (($offset < $comapos)
+                     && (($slashpos = strpos($machine, "/", $offset)) !== false)
+                     && ($slashpos < $comapos)) {
+                    $len1 = $comapos - $slashpos - 1;
+                    $str1 = substr($machine, $slashpos + 1, $len1);
+                    $begstr  = substr($machine, 0, $slashpos);
+                    if ($len1 > 0) { // no empty
+                        $str2 = substr($begstr, -$len1 - 1);
+                    } else {
+                        $str2 = " ";
+                    }
+                    if ((" ".$str1 === $str2) || ($str1 === $begstr)) { // duplicates
+                        $machine = $begstr.$endstr;
+                        break;
+                    }
+                    $offset = $slashpos + 1;
+                }
+            }
+
+            if ($machine != "") {
+                $hardware->addAttribute('Name', $machine);
             }
         }
-        $scsi = null;
-        foreach (System::removeDupsAndCount($this->_sys->getScsiDevices()) as $dev) {
-            if ($scsi === null) $scsi = $hardware->addChild('SCSI');
-            $tmp = $scsi->addChild('Device');
-            $tmp->addAttribute('Name', $dev->getName());
-            $tmp->addAttribute('Count', $dev->getCount());
-            if ($dev->getCapacity() !== null) {
-                $tmp->addAttribute('Capacity', $dev->getCapacity());
+
+        if (defined('PSI_SHOW_VIRTUALIZER_INFO') && PSI_SHOW_VIRTUALIZER_INFO) {
+            $virt = $this->_sys->getVirtualizer();
+            $first = true;
+            $virtstring = "";
+            foreach ($virt as $virtkey=>$virtvalue) if ($virtvalue) {
+                if ($first) {
+                    $first = false;
+                } else {
+                    $virtstring .= ", ";
+                }
+                if ($virtkey === 'microsoft') {
+                    $virtstring .= 'hyper-v';
+                } elseif ($virtkey === 'kvm') {
+                    $virtstring .= 'qemu-kvm';
+                } elseif ($virtkey === 'oracle') {
+                    $virtstring .= 'virtualbox';
+                } elseif ($virtkey === 'zvm') {
+                    $virtstring .= 'z/vm';
+                } else {
+                    $virtstring .= $virtkey;
+                }
+            }
+            if ($virtstring !== "") {
+                $hardware->addAttribute('Virtualizer', $virtstring);
             }
-        }
-        $tb = null;
-        foreach (System::removeDupsAndCount($this->_sys->getTbDevices()) as $dev) {
-            if ($tb === null) $tb = $hardware->addChild('TB');
-            $tmp = $tb->addChild('Device');
-            $tmp->addAttribute('Name', $dev->getName());
-            $tmp->addAttribute('Count', $dev->getCount());
-        }
-        $i2c = null;
-        foreach (System::removeDupsAndCount($this->_sys->getI2cDevices()) as $dev) {
-            if ($i2c === null) $i2c = $hardware->addChild('I2C');
-            $tmp = $i2c->addChild('Device');
-            $tmp->addAttribute('Name', $dev->getName());
-            $tmp->addAttribute('Count', $dev->getCount());
         }
 
         $cpu = null;
+        $vendortab = null;
         foreach ($this->_sys->getCpus() as $oneCpu) {
             if ($cpu === null) $cpu = $hardware->addChild('CPU');
             $tmp = $cpu->addChild('CpuCore');
             $tmp->addAttribute('Model', $oneCpu->getModel());
-            if ($oneCpu->getCpuSpeed() !== 0) {
-                $tmp->addAttribute('CpuSpeed', $oneCpu->getCpuSpeed());
+            if ($oneCpu->getVoltage() > 0) {
+                $tmp->addAttribute('Voltage', $oneCpu->getVoltage());
             }
-            if ($oneCpu->getCpuSpeedMax() !== 0) {
+            if ($oneCpu->getCpuSpeed() > 0) {
+                $tmp->addAttribute('CpuSpeed', $oneCpu->getCpuSpeed());
+            } elseif ($oneCpu->getCpuSpeed() == -1) {
+                $tmp->addAttribute('CpuSpeed', 0); // core stopped
+            }
+            if ($oneCpu->getCpuSpeedMax() > 0) {
                 $tmp->addAttribute('CpuSpeedMax', $oneCpu->getCpuSpeedMax());
             }
-            if ($oneCpu->getCpuSpeedMin() !== 0) {
+            if ($oneCpu->getCpuSpeedMin() > 0) {
                 $tmp->addAttribute('CpuSpeedMin', $oneCpu->getCpuSpeedMin());
             }
 /*
@@ -294,6 +336,13 @@ class XML
             if ($oneCpu->getVirt() !== null) {
                 $tmp->addAttribute('Virt', $oneCpu->getVirt());
             }
+            if ($oneCpu->getVendorId() !== null) {
+                if ($vendortab === null) $vendortab = @parse_ini_file(PSI_APP_ROOT."/data/cpus.ini", true);
+                $shortvendorid = $oneCpu->getVendorId();
+                if ($vendortab && ($shortvendorid != "") && isset($vendortab['manufacturer'][$shortvendorid])) {
+                    $tmp->addAttribute('Manufacturer', $vendortab['manufacturer'][$shortvendorid]);
+                }
+            }
             if ($oneCpu->getBogomips() !== null) {
                 $tmp->addAttribute('Bogomips', $oneCpu->getBogomips());
             }
@@ -301,6 +350,144 @@ class XML
                 $tmp->addAttribute('Load', $oneCpu->getLoad());
             }
         }
+        $mem = null;
+        foreach (System::removeDupsAndCount($this->_sys->getMemDevices()) as $dev) {
+            if ($mem === null) $mem = $hardware->addChild('MEM');
+            $tmp = $mem->addChild('Chip');
+            $tmp->addAttribute('Name', $dev->getName());
+            if (defined('PSI_SHOW_DEVICES_INFOS') && PSI_SHOW_DEVICES_INFOS) {
+                if ($dev->getCapacity() !== null) {
+                    $tmp->addAttribute('Capacity', $dev->getCapacity());
+                }
+                if ($dev->getManufacturer() !== null) {
+                    $tmp->addAttribute('Manufacturer', $dev->getManufacturer());
+                }
+                if ($dev->getProduct() !== null) {
+                    $tmp->addAttribute('Product', $dev->getProduct());
+                }
+                if ($dev->getSpeed() !== null) {
+                    $tmp->addAttribute('Speed', $dev->getSpeed());
+                }
+                if ($dev->getVoltage() !== null) {
+                    $tmp->addAttribute('Voltage', $dev->getVoltage());
+                }
+                if (defined('PSI_SHOW_DEVICES_SERIAL') && PSI_SHOW_DEVICES_SERIAL && ($dev->getSerial() !== null)) {
+                    $tmp->addAttribute('Serial', $dev->getSerial());
+                }
+            }
+            if ($dev->getCount() > 1) {
+                $tmp->addAttribute('Count', $dev->getCount());
+            }
+        }
+        $pci = null;
+        foreach (System::removeDupsAndCount($this->_sys->getPciDevices()) as $dev) {
+            if ($pci === null) $pci = $hardware->addChild('PCI');
+            $tmp = $pci->addChild('Device');
+            $tmp->addAttribute('Name', $dev->getName());
+            if (defined('PSI_SHOW_DEVICES_INFOS') && PSI_SHOW_DEVICES_INFOS) {
+                if ($dev->getManufacturer() !== null) {
+                    $tmp->addAttribute('Manufacturer', $dev->getManufacturer());
+                }
+                if ($dev->getProduct() !== null) {
+                    $tmp->addAttribute('Product', $dev->getProduct());
+                }
+            }
+            if ($dev->getCount() > 1) {
+                $tmp->addAttribute('Count', $dev->getCount());
+            }
+        }
+        $ide = null;
+        foreach (System::removeDupsAndCount($this->_sys->getIdeDevices()) as $dev) {
+            if ($ide === null) $ide = $hardware->addChild('IDE');
+            $tmp = $ide->addChild('Device');
+            $tmp->addAttribute('Name', $dev->getName());
+            if (defined('PSI_SHOW_DEVICES_INFOS') && PSI_SHOW_DEVICES_INFOS) {
+                if ($dev->getCapacity() !== null) {
+                    $tmp->addAttribute('Capacity', $dev->getCapacity());
+                }
+                if (defined('PSI_SHOW_DEVICES_SERIAL') && PSI_SHOW_DEVICES_SERIAL && ($dev->getSerial() !== null)) {
+                    $tmp->addAttribute('Serial', $dev->getSerial());
+                }
+            }
+            if ($dev->getCount() > 1) {
+                $tmp->addAttribute('Count', $dev->getCount());
+            }
+        }
+        $scsi = null;
+        foreach (System::removeDupsAndCount($this->_sys->getScsiDevices()) as $dev) {
+            if ($scsi === null) $scsi = $hardware->addChild('SCSI');
+            $tmp = $scsi->addChild('Device');
+            $tmp->addAttribute('Name', $dev->getName());
+            if (defined('PSI_SHOW_DEVICES_INFOS') && PSI_SHOW_DEVICES_INFOS) {
+                if ($dev->getCapacity() !== null) {
+                    $tmp->addAttribute('Capacity', $dev->getCapacity());
+                }
+                if (defined('PSI_SHOW_DEVICES_SERIAL') && PSI_SHOW_DEVICES_SERIAL && ($dev->getSerial() !== null)) {
+                    $tmp->addAttribute('Serial', $dev->getSerial());
+                }
+            }
+            if ($dev->getCount() > 1) {
+                $tmp->addAttribute('Count', $dev->getCount());
+            }
+        }
+        $nvme = null;
+        foreach (System::removeDupsAndCount($this->_sys->getNvmeDevices()) as $dev) {
+            if ($nvme === null) $nvme = $hardware->addChild('NVMe');
+            $tmp = $nvme->addChild('Device');
+            $tmp->addAttribute('Name', $dev->getName());
+            if (defined('PSI_SHOW_DEVICES_INFOS') && PSI_SHOW_DEVICES_INFOS) {
+                if ($dev->getCapacity() !== null) {
+                    $tmp->addAttribute('Capacity', $dev->getCapacity());
+                }
+                if (defined('PSI_SHOW_DEVICES_SERIAL') && PSI_SHOW_DEVICES_SERIAL && ($dev->getSerial() !== null)) {
+                    $tmp->addAttribute('Serial', $dev->getSerial());
+                }
+            }
+            if ($dev->getCount() > 1) {
+                $tmp->addAttribute('Count', $dev->getCount());
+            }
+        }
+        $usb = null;
+        foreach (System::removeDupsAndCount($this->_sys->getUsbDevices()) as $dev) {
+            if ($usb === null) $usb = $hardware->addChild('USB');
+            $tmp = $usb->addChild('Device');
+            $tmp->addAttribute('Name', $dev->getName());
+            if (defined('PSI_SHOW_DEVICES_INFOS') && PSI_SHOW_DEVICES_INFOS) {
+                if ($dev->getManufacturer() !== null) {
+                    $tmp->addAttribute('Manufacturer', $dev->getManufacturer());
+                }
+                if ($dev->getProduct() !== null) {
+                    $tmp->addAttribute('Product', $dev->getProduct());
+                }
+                if ($dev->getSpeed() !== null) {
+                    $tmp->addAttribute('Speed', $dev->getSpeed());
+                }
+                if (defined('PSI_SHOW_DEVICES_SERIAL') && PSI_SHOW_DEVICES_SERIAL && ($dev->getSerial() !== null)) {
+                    $tmp->addAttribute('Serial', $dev->getSerial());
+                }
+            }
+            if ($dev->getCount() > 1) {
+                $tmp->addAttribute('Count', $dev->getCount());
+            }
+        }
+        $tb = null;
+        foreach (System::removeDupsAndCount($this->_sys->getTbDevices()) as $dev) {
+            if ($tb === null) $tb = $hardware->addChild('TB');
+            $tmp = $tb->addChild('Device');
+            $tmp->addAttribute('Name', $dev->getName());
+            if ($dev->getCount() > 1) {
+                $tmp->addAttribute('Count', $dev->getCount());
+            }
+        }
+        $i2c = null;
+        foreach (System::removeDupsAndCount($this->_sys->getI2cDevices()) as $dev) {
+            if ($i2c === null) $i2c = $hardware->addChild('I2C');
+            $tmp = $i2c->addChild('Device');
+            $tmp->addAttribute('Name', $dev->getName());
+            if ($dev->getCount() > 1) {
+                $tmp->addAttribute('Count', $dev->getCount());
+            }
+        }
     }
 
     /**
@@ -349,28 +536,32 @@ class XML
      *
      * @param SimpleXmlExtended $mount Xml-Element
      * @param DiskDevice        $dev   DiskDevice
-     * @param Integer           $i     counter
+     * @param int               $i     counter
      *
-     * @return Void
+     * @return void
      */
     private function _fillDevice(SimpleXMLExtended $mount, DiskDevice $dev, $i)
     {
         $mount->addAttribute('MountPointID', $i);
-        if ($dev->getFsType()!=="") $mount->addAttribute('FSType', $dev->getFsType());
+        if ($dev->getFsType()!=="") {
+            $mount->addAttribute('FSType', $dev->getFsType());
+        }
         $mount->addAttribute('Name', $dev->getName());
         $mount->addAttribute('Free', sprintf("%.0f", $dev->getFree()));
         $mount->addAttribute('Used', sprintf("%.0f", $dev->getUsed()));
         $mount->addAttribute('Total', sprintf("%.0f", $dev->getTotal()));
-        $mount->addAttribute('Percent', $dev->getPercentUsed());
-        if (PSI_SHOW_MOUNT_OPTION === true) {
+        $percentUsed = $dev->getPercentUsed();
+        $mount->addAttribute('Percent', $percentUsed);
+        if ($dev->getPercentInodesUsed() !== null) {
+            $mount->addAttribute('Inodes', $dev->getPercentInodesUsed());
+        }
+        if ($dev->getIgnore() > 0) $mount->addAttribute('Ignore', $dev->getIgnore());
+        if (PSI_SHOW_MOUNT_OPTION) {
             if ($dev->getOptions() !== null) {
                 $mount->addAttribute('MountOptions', preg_replace("/,/", ", ", $dev->getOptions()));
             }
         }
-        if ($dev->getPercentInodesUsed() !== null) {
-            $mount->addAttribute('Inodes', $dev->getPercentInodesUsed());
-        }
-        if (PSI_SHOW_MOUNT_POINT === true) {
+        if (PSI_SHOW_MOUNT_POINT && ($dev->getMountPoint() !== null)) {
             $mount->addAttribute('MountPoint', $dev->getMountPoint());
         }
     }
@@ -382,8 +573,7 @@ class XML
      */
     private function _buildFilesystems()
     {
-        $hideMounts = $hideFstypes = $hideDisks = array();
-        $i = 1;
+        $hideMounts = $hideFstypes = $hideDisks = $ignoreFree = $ignoreTotal = $ignoreUsage = $ignoreThreshold = array();
         if (defined('PSI_HIDE_MOUNTS') && is_string(PSI_HIDE_MOUNTS)) {
             if (preg_match(ARRAY_EXP, PSI_HIDE_MOUNTS)) {
                 $hideMounts = eval(PSI_HIDE_MOUNTS);
@@ -409,10 +599,48 @@ class XML
                 return;
             }
         }
+        if (defined('PSI_IGNORE_FREE') && is_string(PSI_IGNORE_FREE)) {
+            if (preg_match(ARRAY_EXP, PSI_IGNORE_FREE)) {
+                $ignoreFree = eval(PSI_IGNORE_FREE);
+            } else {
+                $ignoreFree = array(PSI_IGNORE_FREE);
+            }
+        }
+        if (defined('PSI_IGNORE_TOTAL') && is_string(PSI_IGNORE_TOTAL)) {
+            if (preg_match(ARRAY_EXP, PSI_IGNORE_TOTAL)) {
+                $ignoreTotal = eval(PSI_IGNORE_TOTAL);
+            } else {
+                $ignoreTotal = array(PSI_IGNORE_TOTAL);
+            }
+        }
+        if (defined('PSI_IGNORE_USAGE') && is_string(PSI_IGNORE_USAGE)) {
+            if (preg_match(ARRAY_EXP, PSI_IGNORE_USAGE)) {
+                $ignoreUsage = eval(PSI_IGNORE_USAGE);
+            } else {
+                $ignoreUsage = array(PSI_IGNORE_USAGE);
+            }
+        }
+        if (defined('PSI_IGNORE_THRESHOLD_FS_TYPES') && is_string(PSI_IGNORE_THRESHOLD_FS_TYPES)) {
+            if (preg_match(ARRAY_EXP, PSI_IGNORE_THRESHOLD_FS_TYPES)) {
+                $ignoreThreshold = eval(PSI_IGNORE_THRESHOLD_FS_TYPES);
+            } else {
+                $ignoreThreshold = array(PSI_IGNORE_THRESHOLD_FS_TYPES);
+            }
+        }
         $fs = $this->_xml->addChild('FileSystem');
+        $i = 1;
         foreach ($this->_sys->getDiskDevices() as $disk) {
             if (!in_array($disk->getMountPoint(), $hideMounts, true) && !in_array($disk->getFsType(), $hideFstypes, true) && !in_array($disk->getName(), $hideDisks, true)) {
                 $mount = $fs->addChild('Mount');
+                if (in_array($disk->getFsType(), $ignoreThreshold, true)) {
+                    $disk->setIgnore(4);
+                } elseif (in_array($disk->getMountPoint(), $ignoreUsage, true)) {
+                    $disk->setIgnore(3);
+                } elseif (in_array($disk->getMountPoint(), $ignoreTotal, true)) {
+                    $disk->setIgnore(2);
+                } elseif (in_array($disk->getMountPoint(), $ignoreFree, true)) {
+                    $disk->setIgnore(1);
+                }
                 $this->_fillDevice($mount, $disk, $i++);
             }
         }
@@ -433,96 +661,118 @@ class XML
                 $mbinfo_data = new $mbinfoclass();
                 $mbinfo_detail = $mbinfo_data->getMBInfo();
 
-                if (!defined('PSI_ONLY') || PSI_ONLY==='temperature') foreach ($mbinfo_detail->getMbTemp() as $dev) {
+                if (!$this->_sysinfo->getBlockName() || $this->_sysinfo->getBlockName()==='temperature') foreach ($mbinfo_detail->getMbTemp() as $dev) {
                     if ($temp == null) {
                         $temp = $mbinfo->addChild('Temperature');
                     }
                     $item = $temp->addChild('Item');
                     $item->addAttribute('Label', $dev->getName());
                     $item->addAttribute('Value', $dev->getValue());
+                    $alarm = false;
                     if ($dev->getMax() !== null) {
                         $item->addAttribute('Max', $dev->getMax());
+                        $alarm = true;
                     }
-                    if (defined('PSI_SENSOR_EVENTS') && PSI_SENSOR_EVENTS && $dev->getEvent() !== "") {
-                        $item->addAttribute('Event', $dev->getEvent());
+                    if (defined('PSI_SENSOR_EVENTS') && PSI_SENSOR_EVENTS && ($dev->getEvent() !== "") && (((strtolower($dev->getEvent())) !== "alarm") || $alarm || ($dev->getValue() == 0))) {
+                        $item->addAttribute('Event', ucfirst(strtolower($dev->getEvent())));
                     }
                 }
 
-                if (!defined('PSI_ONLY') || PSI_ONLY==='fans') foreach ($mbinfo_detail->getMbFan() as $dev) {
+                if (!$this->_sysinfo->getBlockName() || $this->_sysinfo->getBlockName()==='fans') foreach ($mbinfo_detail->getMbFan() as $dev) {
                     if ($fan == null) {
                         $fan = $mbinfo->addChild('Fans');
                     }
                     $item = $fan->addChild('Item');
                     $item->addAttribute('Label', $dev->getName());
                     $item->addAttribute('Value', $dev->getValue());
+                    $alarm = false;
                     if ($dev->getMin() !== null) {
                         $item->addAttribute('Min', $dev->getMin());
+                        $alarm = true;
                     }
-                    if (defined('PSI_SENSOR_EVENTS') && PSI_SENSOR_EVENTS && $dev->getEvent() !== "") {
-                        $item->addAttribute('Event', $dev->getEvent());
+                    if ($dev->getUnit() !== "") {
+                        $item->addAttribute('Unit', $dev->getUnit());
+                    }
+                    if (defined('PSI_SENSOR_EVENTS') && PSI_SENSOR_EVENTS && ($dev->getEvent() !== "") && (((strtolower($dev->getEvent())) !== "alarm") || $alarm || ($dev->getValue() == 0))) {
+                        $item->addAttribute('Event', ucfirst(strtolower($dev->getEvent())));
                     }
                 }
 
-                if (!defined('PSI_ONLY') || PSI_ONLY==='voltage') foreach ($mbinfo_detail->getMbVolt() as $dev) {
+                if (!$this->_sysinfo->getBlockName() || $this->_sysinfo->getBlockName()==='voltage') foreach ($mbinfo_detail->getMbVolt() as $dev) {
                     if ($volt == null) {
                         $volt = $mbinfo->addChild('Voltage');
                     }
                     $item = $volt->addChild('Item');
                     $item->addAttribute('Label', $dev->getName());
                     $item->addAttribute('Value', $dev->getValue());
-                    if ($dev->getMin() !== null) {
-                        $item->addAttribute('Min', $dev->getMin());
+                    $alarm = false;
+                    if (($dev->getMin() === null) || ($dev->getMin() != 0) || ($dev->getMax() === null) || ($dev->getMax() != 0)) {
+                        if ($dev->getMin() !== null) {
+                            $item->addAttribute('Min', $dev->getMin());
+                            $alarm = true;
+                        }
+                        if ($dev->getMax() !== null) {
+                            $item->addAttribute('Max', $dev->getMax());
+                            $alarm = true;
+                        }
                     }
-                    if ($dev->getMax() !== null) {
-                        $item->addAttribute('Max', $dev->getMax());
-                    }
-                    if (defined('PSI_SENSOR_EVENTS') && PSI_SENSOR_EVENTS && $dev->getEvent() !== "") {
-                        $item->addAttribute('Event', $dev->getEvent());
+                    if (defined('PSI_SENSOR_EVENTS') && PSI_SENSOR_EVENTS && ($dev->getEvent() !== "") && (((strtolower($dev->getEvent())) !== "alarm") || $alarm || ($dev->getValue() == 0))) {
+                        $item->addAttribute('Event', ucfirst(strtolower($dev->getEvent())));
                     }
                 }
 
-                if (!defined('PSI_ONLY') || PSI_ONLY==='power') foreach ($mbinfo_detail->getMbPower() as $dev) {
+                if (!$this->_sysinfo->getBlockName() || $this->_sysinfo->getBlockName()==='power') foreach ($mbinfo_detail->getMbPower() as $dev) {
                     if ($power == null) {
                         $power = $mbinfo->addChild('Power');
                     }
                     $item = $power->addChild('Item');
                     $item->addAttribute('Label', $dev->getName());
                     $item->addAttribute('Value', $dev->getValue());
+                    $alarm = false;
                     if ($dev->getMax() !== null) {
                         $item->addAttribute('Max', $dev->getMax());
+                        $alarm = true;
                     }
-                    if (defined('PSI_SENSOR_EVENTS') && PSI_SENSOR_EVENTS && $dev->getEvent() !== "") {
-                        $item->addAttribute('Event', $dev->getEvent());
+                    if (defined('PSI_SENSOR_EVENTS') && PSI_SENSOR_EVENTS && ($dev->getEvent() !== "") && (((strtolower($dev->getEvent())) !== "alarm") || $alarm || ($dev->getValue() == 0))) {
+                        $item->addAttribute('Event', ucfirst(strtolower($dev->getEvent())));
                     }
                 }
 
-                if (!defined('PSI_ONLY') || PSI_ONLY==='current') foreach ($mbinfo_detail->getMbCurrent() as $dev) {
+                if (!$this->_sysinfo->getBlockName() || $this->_sysinfo->getBlockName()==='current') foreach ($mbinfo_detail->getMbCurrent() as $dev) {
                     if ($current == null) {
                         $current = $mbinfo->addChild('Current');
                     }
                     $item = $current->addChild('Item');
                     $item->addAttribute('Label', $dev->getName());
                     $item->addAttribute('Value', $dev->getValue());
-                    if ($dev->getMin() !== null) {
-                        $item->addAttribute('Min', $dev->getMin());
+                    $alarm = false;
+                    if (($dev->getMin() === null) || ($dev->getMin() != 0) || ($dev->getMax() === null) || ($dev->getMax() != 0)) {
+                        if ($dev->getMin() !== null) {
+                            $item->addAttribute('Min', $dev->getMin());
+                            $alarm = true;
+                        }
+                        if ($dev->getMax() !== null) {
+                            $item->addAttribute('Max', $dev->getMax());
+                            $alarm = true;
+                        }
                     }
-                    if ($dev->getMax() !== null) {
-                        $item->addAttribute('Max', $dev->getMax());
-                    }
-                    if (defined('PSI_SENSOR_EVENTS') && PSI_SENSOR_EVENTS && $dev->getEvent() !== "") {
-                        $item->addAttribute('Event', $dev->getEvent());
+                    if (defined('PSI_SENSOR_EVENTS') && PSI_SENSOR_EVENTS && ($dev->getEvent() !== "") && (((strtolower($dev->getEvent())) !== "alarm") || $alarm || ($dev->getValue() == 0))) {
+                        $item->addAttribute('Event', ucfirst(strtolower($dev->getEvent())));
                     }
                 }
 
-                if (!defined('PSI_ONLY') || PSI_ONLY==='other') foreach ($mbinfo_detail->getMbOther() as $dev) {
+                if (!$this->_sysinfo->getBlockName() || $this->_sysinfo->getBlockName()==='other') foreach ($mbinfo_detail->getMbOther() as $dev) {
                     if ($other == null) {
                         $other = $mbinfo->addChild('Other');
                     }
                     $item = $other->addChild('Item');
                     $item->addAttribute('Label', $dev->getName());
                     $item->addAttribute('Value', $dev->getValue());
+                    if ($dev->getUnit() !== "") {
+                        $item->addAttribute('Unit', $dev->getUnit());
+                    }
                     if (defined('PSI_SENSOR_EVENTS') && PSI_SENSOR_EVENTS && $dev->getEvent() !== "") {
-                        $item->addAttribute('Event', $dev->getEvent());
+                        $item->addAttribute('Event', ucfirst(strtolower($dev->getEvent())));
                     }
                 }
             }
@@ -537,7 +787,7 @@ class XML
     private function _buildUpsinfo()
     {
         $upsinfo = $this->_xml->addChild('UPSInfo');
-        if (defined('PSI_UPS_APCUPSD_CGI_ENABLE') && PSI_UPS_APCUPSD_CGI_ENABLE) {
+        if (!defined('PSI_EMU_HOSTNAME') && defined('PSI_UPS_APCUPSD_CGI_ENABLE') && PSI_UPS_APCUPSD_CGI_ENABLE) {
             $upsinfo->addAttribute('ApcupsdCgiLinks', true);
         }
         if (sizeof(unserialize(PSI_UPSINFO))>0) {
@@ -557,6 +807,9 @@ class XML
                         $item->addAttribute('StartTime', $ups->getStartTime());
                     }
                     $item->addAttribute('Status', $ups->getStatus());
+                    if ($ups->getBeeperStatus() !== null) {
+                        $item->addAttribute('BeeperStatus', $ups->getBeeperStatus());
+                    }
                     if ($ups->getTemperatur() !== null) {
                         $item->addAttribute('Temperature', $ups->getTemperatur());
                     }
@@ -602,12 +855,12 @@ class XML
      */
     private function _buildXml()
     {
-        if (!$this->_plugin_request || $this->_complete_request) {
+        if (($this->_plugin == '') || $this->_complete_request) {
             if ($this->_sys === null) {
-                if (PSI_DEBUG === true) {
+                if (PSI_DEBUG) {
                     // unstable version check
                     if (!is_numeric(substr(PSI_VERSION, -1))) {
-                        $this->_errors->addError("WARN", "This is an unstable version of phpSysInfo, some things may not work correctly");
+                        $this->_errors->addWarning("This is an unstable version of phpSysInfo, some things may not work correctly");
                     }
 
                     // Safe mode check
@@ -627,21 +880,21 @@ class XML
                         $this->_errors->addError("WARN", "PhpSysInfo requires '.' inside the 'include_path' in php.ini");
                     }
                     // popen mode check
-                    if (defined("PSI_MODE_POPEN") && PSI_MODE_POPEN === true) {
+                    if (defined("PSI_MODE_POPEN") && PSI_MODE_POPEN) {
                         $this->_errors->addError("WARN", "Installed version of PHP does not support proc_open() function, popen() is used");
                     }
                 }
                 $this->_sys = $this->_sysinfo->getSys();
             }
-            if (!defined('PSI_ONLY') || PSI_ONLY==='vitals') $this->_buildVitals();
-            if (!defined('PSI_ONLY') || PSI_ONLY==='network') $this->_buildNetwork();
-            if (!defined('PSI_ONLY') || PSI_ONLY==='hardware') $this->_buildHardware();
-            if (!defined('PSI_ONLY') || PSI_ONLY==='memory') $this->_buildMemory();
-            if (!defined('PSI_ONLY') || PSI_ONLY==='filesystem') $this->_buildFilesystems();
-            if (!defined('PSI_ONLY') || in_array(PSI_ONLY, array('voltage','current','temperature','fans','power','other'))) $this->_buildMbinfo();
-            if (!defined('PSI_ONLY') || PSI_ONLY==='ups') $this->_buildUpsinfo();
+            if (!$this->_sysinfo->getBlockName() || $this->_sysinfo->getBlockName()==='vitals') $this->_buildVitals();
+            if (!$this->_sysinfo->getBlockName() || $this->_sysinfo->getBlockName()==='network') $this->_buildNetwork();
+            if (!$this->_sysinfo->getBlockName() || $this->_sysinfo->getBlockName()==='hardware') $this->_buildHardware();
+            if (!$this->_sysinfo->getBlockName() || $this->_sysinfo->getBlockName()==='memory') $this->_buildMemory();
+            if (!$this->_sysinfo->getBlockName() || $this->_sysinfo->getBlockName()==='filesystem') $this->_buildFilesystems();
+            if (!$this->_sysinfo->getBlockName() || in_array($this->_sysinfo->getBlockName(), array('voltage','current','temperature','fans','power','other'))) $this->_buildMbinfo();
+            if (!$this->_sysinfo->getBlockName() || $this->_sysinfo->getBlockName()==='ups') $this->_buildUpsinfo();
         }
-        if (!defined('PSI_ONLY')) $this->_buildPlugins();
+        if (!$this->_sysinfo->getBlockName()) $this->_buildPlugins();
         $this->_xml->combinexml($this->_errors->errorsAddToXML($this->_sysinfo->getEncoding()));
     }
 
@@ -665,20 +918,23 @@ class XML
     private function _buildPlugins()
     {
         $pluginroot = $this->_xml->addChild("Plugins");
-        if (($this->_plugin_request || $this->_complete_request) && count($this->_plugins) > 0) {
+        if ((($this->_plugin != '') || $this->_complete_request) && count($this->_plugins) > 0) {
             $plugins = array();
             if ($this->_complete_request) {
                 $plugins = $this->_plugins;
             }
-            if ($this->_plugin_request) {
+            if (($this->_plugin != '')) {
                 $plugins = array($this->_plugin);
             }
             foreach ($plugins as $plugin) {
-                $object = new $plugin($this->_sysinfo->getEncoding());
-                $object->execute();
-                $oxml = $object->xml();
-                if (sizeof($oxml) > 0) {
-                    $pluginroot->combinexml($oxml);
+                if (!$this->_complete_request || !defined('PSI_PLUGIN_'.strtoupper($plugin).'_WMI_HOSTNAME') ||
+                   (defined('PSI_WMI_HOSTNAME') && (PSI_WMI_HOSTNAME == constant('PSI_PLUGIN_'.strtoupper($plugin).'_WMI_HOSTNAME')))) {
+                    $object = new $plugin($this->_sysinfo->getEncoding());
+                    $object->execute();
+                    $oxml = $object->xml();
+                    if (sizeof($oxml) > 0) {
+                        $pluginroot->combinexml($oxml);
+                    }
                 }
             }
         }
@@ -707,27 +963,19 @@ class XML
         $options->addAttribute('byteFormat', defined('PSI_BYTE_FORMAT') ? strtolower(PSI_BYTE_FORMAT) : 'auto_binary');
         $options->addAttribute('datetimeFormat', defined('PSI_DATETIME_FORMAT') ? strtolower(PSI_DATETIME_FORMAT) : 'utc');
         if (defined('PSI_REFRESH')) {
-            if (PSI_REFRESH === false) {
-                $options->addAttribute('refresh', 0);
-            } elseif (PSI_REFRESH === true) {
-                $options->addAttribute('refresh', 1);
-            } else {
-                $options->addAttribute('refresh', PSI_REFRESH);
-            }
+            $options->addAttribute('refresh', max(intval(PSI_REFRESH), 0));
         } else {
             $options->addAttribute('refresh', 60000);
         }
         if (defined('PSI_FS_USAGE_THRESHOLD')) {
-            if (PSI_FS_USAGE_THRESHOLD === true) {
-                $options->addAttribute('threshold', 1);
-            } elseif ((PSI_FS_USAGE_THRESHOLD !== false) && (PSI_FS_USAGE_THRESHOLD >= 1) && (PSI_FS_USAGE_THRESHOLD <= 99)) {
-                $options->addAttribute('threshold', PSI_FS_USAGE_THRESHOLD);
+            if ((($fsut = intval(PSI_FS_USAGE_THRESHOLD)) >= 1) && ($fsut <= 99)) {
+                $options->addAttribute('threshold', $fsut);
             }
         } else {
             $options->addAttribute('threshold', 90);
         }
         if (count($this->_plugins) > 0) {
-            if ($this->_plugin_request) {
+            if (($this->_plugin != '')) {
                 $plug = $this->_xml->addChild('UsedPlugins');
                 $plug->addChild('Plugin')->addAttribute('name', $this->_plugin);
             } elseif ($this->_complete_request) {
