@@ -4,35 +4,35 @@ function renderPlugin_quotas(data) {
 
         ByteUsed: {
             html: function () {
-                return formatBytes(this["ByteUsed"], data["Options"]["@attributes"]["byteFormat"]);
+                return formatBytes(this.ByteUsed, data.Options["@attributes"].byteFormat);
             }
         },
         ByteSoft: {
             html: function () {
-                return formatBytes(this["ByteSoft"], data["Options"]["@attributes"]["byteFormat"]);
+                return formatBytes(this.ByteSoft, data.Options["@attributes"].byteFormat);
             }
         },
         ByteHard: {
             html: function () {
-                return formatBytes(this["ByteHard"], data["Options"]["@attributes"]["byteFormat"]);
+                return formatBytes(this.ByteHard, data.Options["@attributes"].byteFormat);
             }
         },
         BytePercentUsed: {
             html: function () {
-                return '<div class="progress"><div class="progress-bar progress-bar-info" style="width:' + this["BytePercentUsed"] + '%;"></div>' +
-                        '</div><div class="percent">' + this["BytePercentUsed"] + '%</div>';
+                return '<div class="progress"><div class="progress-bar progress-bar-info" style="width:' + this.BytePercentUsed + '%;"></div>' +
+                        '</div><div class="percent">' + this.BytePercentUsed + '%</div>';
             }
         },
         FilePercentUsed: {
             html: function () {
-                return '<div class="progress"><div class="progress-bar progress-bar-info" style="width:' + this["FilePercentUsed"] + '%;"></div>' +
-                        '</div><div class="percent">' + this["FilePercentUsed"] + '%</div>';
+                return '<div class="progress"><div class="progress-bar progress-bar-info" style="width:' + this.FilePercentUsed + '%;"></div>' +
+                        '</div><div class="percent">' + this.FilePercentUsed + '%</div>';
             }
         }
     };
 
-    if (data['Plugins']['Plugin_Quotas'] !== undefined) {
-        var qtitems = items(data['Plugins']['Plugin_Quotas']['Quota']);
+    if (data.Plugins.Plugin_Quotas !== undefined) {
+        var qtitems = items(data.Plugins.Plugin_Quotas.Quota);
         if (qtitems.length > 0) {
             var qt_memory = [];
             qt_memory.push_attrs(qtitems);
