@@ -18,7 +18,7 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 
-/*global $, jQuery, buildBlock, datetime, plugin_translate, genlang, createBar */
+/*global $, jQuery, buildBlock, datetime, plugin_translate, genlang */
 
 "use strict";
 
@@ -28,8 +28,13 @@ var UpdateNotifier_show = false, UpdateNotifier_table;
  * @param {jQuery} xml plugin-XML
  */
 function updatenotifier_populate(xml) {
-    var html = "";
+    var html = "", hostname = "";
  
+    hostname = $("Plugins Plugin_UpdateNotifier", xml).attr('Hostname');
+    if (hostname !== undefined) {
+        $('span[class=Hostname_UpdateNotifier]').html(hostname);
+    }
+
     $("Plugins Plugin_UpdateNotifier UpdateNotifier", xml).each(function(idp) {
         var packages = "", security = "";
         packages = $("packages", this).text();
@@ -38,19 +43,19 @@ function updatenotifier_populate(xml) {
         //UpdateNotifier_table.fnAddData([packages]);
         //UpdateNotifier_table.fnAddData([security]);
 
-        html  = "  <tr>\n";
-        html += "    <td>" + packages + " " + genlang(3, false, "UpdateNotifier") + "</td>\n";
-        html += "  </tr>\n";
-        html += "  <tr>\n";
-        html += "    <td>" + security + " " + genlang(4, false, "UpdateNotifier") + "</td>\n";
-        html += "  </tr>\n";
+        html  = "    <tr>\n";
+        html += "      <td>" + packages + " " + genlang(3, "UpdateNotifier") + "</td>\n";
+        html += "    </tr>\n";
+        html += "    <tr>\n";
+        html += "      <td>" + security + " " + genlang(4, "UpdateNotifier") + "</td>\n";
+        html += "    </tr>\n";
 
         $("#Plugin_UpdateNotifier tbody").empty().append(html);
 
-        if ((packages == 0) && (security == 0)) {
-            $("#UpdateNotifierTable-info").html(genlang(5, false, "UpdateNotifier"));
+        if ((packages <= 0) && (security <= 0)) {
+            $("#UpdateNotifierTable-info").html(genlang(5, "UpdateNotifier"));
         } else {
-            $("#UpdateNotifierTable-info").html(genlang(2, false, "UpdateNotifier"));
+            $("#UpdateNotifierTable-info").html(genlang(2, "UpdateNotifier"));
         }
 
         UpdateNotifier_show = true;
@@ -63,15 +68,17 @@ function updatenotifier_populate(xml) {
 function updatenotifier_buildTable() {
     var html = "";
 
-    html += "<table id=\"Plugin_UpdateNotifierTable\" style=\"border-spacing:0;\">\n";
-    html += "  <thead>\n";
-    html += "    <tr>\n";
-    html += "      <th id=\"UpdateNotifierTable-info\">" + genlang(2, false, "UpdateNotifier") + "</th>\n";
-    html += "    </tr>\n";
-    html += "  </thead>\n";
-    html += "  <tbody>\n";
-    html += "  </tbody>\n";
-    html += "</table>\n";
+    html += "<div style=\"overflow-x:auto;\">\n";
+    html += "  <table id=\"Plugin_UpdateNotifierTable\" style=\"border-collapse:collapse;\">\n";
+    html += "    <thead>\n";
+    html += "      <tr>\n";
+    html += "        <th id=\"UpdateNotifierTable-info\">" + genlang(2, "UpdateNotifier") + "</th>\n";
+    html += "      </tr>\n";
+    html += "    </thead>\n";
+    html += "    <tbody>\n";
+    html += "    </tbody>\n";
+    html += "  </table>\n";
+    html += "</div>\n";
 
     $("#Plugin_UpdateNotifier").append(html);
 
@@ -101,7 +108,7 @@ function updatenotifier_request() {
 
 $(document).ready(function() {
     $("#footer").before(buildBlock("UpdateNotifier", 1, true));
-    $("#Plugin_UpdateNotifier").css("width", "451px");
+    $("#Plugin_UpdateNotifier").addClass("halfsize");
 
     updatenotifier_buildTable();
     updatenotifier_request();
