@@ -1,21 +1,22 @@
 <?php
 echo "<!DOCTYPE html>";
+echo "<head>";
 echo "<meta charset=\"UTF-8\">";
 echo "<title> </title>";
+echo "</head>";
 echo "<body>";
 
-define('APP_ROOT', dirname(__FILE__).'/..');
-require_once APP_ROOT.'/includes/interface/class.PSI_Interface_OS.inc.php';
-require_once APP_ROOT.'/includes/os/class.OS.inc.php';
-require_once APP_ROOT.'/includes/to/class.System.inc.php';
-require_once APP_ROOT.'/includes/os/class.Linux.inc.php';
-define('PSI_USE_VHOST', false);
+define('PSI_APP_ROOT', dirname(__FILE__).'/..');
 define('PSI_DEBUG', false);
-define('PSI_LOAD_BAR', false);
+require_once PSI_APP_ROOT.'/includes/interface/class.PSI_Interface_OS.inc.php';
+require_once PSI_APP_ROOT.'/includes/os/class.OS.inc.php';
+require_once PSI_APP_ROOT.'/includes/to/class.System.inc.php';
+require_once PSI_APP_ROOT.'/includes/os/class.Linux.inc.php';
 
 $log_file = "";
 $lsb = true; //enable detection lsb_release -a
 $lsbfile = true; //enable detection /etc/lsb-release
+$other = true; //enable other detection
 
 class PSI_Error
 {
@@ -24,18 +25,6 @@ class PSI_Error
     }
 }
 
-class Parser
-{
-    public static function lspci()
-    {
-        return array();
-    }
-    public static function df()
-    {
-        return array();
-    }
-}
-
 class CommonFunctions
 {
     private static function _parse_log_file($string)
@@ -64,9 +53,14 @@ class CommonFunctions
 
     public static function rfts($strFileName, &$strRet, $intLines = 0, $intBytes = 4096, $booErrorRep = true)
     {
-        global $lsb;
         global $lsbfile;
-        if ($lsb || $lsbfile || ($strFileName != "/etc/lsb-release")) {
+        global $other;
+        if ($strFileName=="/etc/lsb-release") {
+            $test = $lsbfile;
+        } else {
+            $test = $other;
+        }
+        if ($test) {
             $strRet=self::_parse_log_file($strFileName);
             if ($strRet && ($intLines == 1) && (strpos($strRet, "\n") !== false)) {
                 $strRet=trim(substr($strRet, 0, strpos($strRet, "\n")));
@@ -78,35 +72,30 @@ class CommonFunctions
         }
     }
 
-    public static function executeProgram($strProgramname, $strArgs, &$strBuffer, $booErrorRep = true)
+    public static function executeProgram($strProgramname, $strArgs, &$strBuffer, $booErrorRep = true, $timeout = 30)
     {
         global $lsb;
+        global $other;
         $strBuffer = '';
         if ($strProgramname=='lsb_release') {
             return $lsb && ($strBuffer = self::_parse_log_file('lsb_release -a'));
         } else {
-            return $strBuffer = self::_parse_log_file($strProgramname);
+            return $other && ($strBuffer = self::_parse_log_file($strProgramname));
         }
     }
 
     public static function fileexists($strFileName)
     {
         global $log_file;
-        global $lsb;
         global $lsbfile;
-        if (file_exists($log_file)
-            && ($lsb || $lsbfile || ($strFileName != "/etc/lsb-release"))
-            && ($contents = @file_get_contents($log_file))
-            && preg_match("/^\-\-\-\-\-\-\-\-\-\-".preg_quote($strFileName, '/')."\-\-\-\-\-\-\-\-\-\-\r?\n/m", $contents)) {
-            return true;
+        global $other;
+        if ($strFileName=="/etc/lsb-release") {
+            $test = $lsbfile;
+        } else {
+            $test = $other;
         }
 
-        return false;
-    }
-
-    public static function gdc()
-    {
-        return array();
+        return $test && file_exists($log_file) && ($contents = @file_get_contents($log_file)) && preg_match("/^\-\-\-\-\-\-\-\-\-\-".preg_quote($strFileName, '/')."\-\-\-\-\-\-\-\-\-\-\r?\n/m", $contents);
     }
 }
 
@@ -118,9 +107,9 @@ class _Linux extends Linux
     }
 }
 
-$system = new _Linux();
-if ($handle = opendir(APP_ROOT.'/sample/distrotest')) {
-    echo "<table cellpadding=\"2\" border=\"1\"  CELLSPACING=\"0\"";
+$system = new _Linux('none');
+if ($handle = opendir(PSI_APP_ROOT.'/sample/distrotest')) {
+    echo "<table cellpadding=\"2\" border=\"1\"  CELLSPACING=\"0\">";
     echo "<tr>";
     echo "<td>Distrotest sample</td>";
     echo "<td>Distro Name</td>";
@@ -132,10 +121,10 @@ if ($handle = opendir(APP_ROOT.'/sample/distrotest')) {
     echo "</tr>";
     while (false !== ($entry = readdir($handle))) {
         if (($entry!=".")&&($entry!="..")) {
-            if ($shandle = opendir(APP_ROOT."/sample/distrotest/$entry")) {
+            if ($shandle = opendir(PSI_APP_ROOT."/sample/distrotest/$entry")) {
                 while (false !== ($sentry = readdir($shandle))) {
                     if (($sentry!=".")&&($sentry!="..")) {
-                        $log_file=APP_ROOT.'/sample/distrotest/'.$entry.'/'.$sentry;
+                        $log_file=PSI_APP_ROOT.'/sample/distrotest/'.$entry.'/'.$sentry;
                         echo "<tr>";
                         echo "<td>".$entry.'/'.$sentry."</td>";
 
@@ -154,7 +143,7 @@ if ($handle = opendir(APP_ROOT.'/sample/distrotest')) {
                             echo "<td style='color:red'>";
                         else
                             echo "<td>";
-                        echo "<img src=\"../gfx/images/".$icon."\" height=\"16\" width=\"16\">";
+                        echo "<img src=\"../gfx/images/".$icon."\" height=\"16\" width=\"16\"/>";
                         echo $icon."</td>";
                         $sys->setDistribution("");
                         $sys->setDistributionIcon("");
@@ -174,7 +163,7 @@ if ($handle = opendir(APP_ROOT.'/sample/distrotest')) {
                             echo "<td style='color:red'>";
                         else
                             echo "<td>";
-                        echo "<img src=\"../gfx/images/".$icon."\" height=\"16\" width=\"16\">";
+                        echo "<img src=\"../gfx/images/".$icon."\" height=\"16\" width=\"16\"/>";
                         echo $icon."</td>";
                         $sys->setDistribution("");
                         $sys->setDistributionIcon("");
@@ -194,7 +183,7 @@ if ($handle = opendir(APP_ROOT.'/sample/distrotest')) {
                             echo "<td style='color:red'>";
                         else
                             echo "<td>";
-                        echo "<img src=\"../gfx/images/".$icon."\" height=\"16\" width=\"16\">";
+                        echo "<img src=\"../gfx/images/".$icon."\" height=\"16\" width=\"16\"/>";
                         echo $icon."</td>";
                         $sys->setDistribution("");
                         $sys->setDistributionIcon("");
